from django.shortcuts import render_to_response
from django import forms
from django.conf import settings
from tasks.models import *
from django.contrib.sites.models import Site
from django.template import RequestContext
from django.http import HttpResponseRedirect, HttpResponse

def task_list(request):
	tasks = Task.objects.all().values('id', 'status', 'name', 'modification_date', 'page', 'author').order_by('-modification_date')
	if len(tasks) == 0:
		return render_to_response('pages/bug.html', {'bug': _('No tasks available. To create a new task click <a href="/tasks/add/">here</a>')}, context_instance=RequestContext(request))
	for i in tasks:
		p = Page.objects.get(id=i['page'])
		if p.slug == 'index':
			i['page'] = _('Global Task')
		else:
			i['page'] = '<a href="/w/p/' + p.slug + '/">' + p.title + '</a>'
	return render_to_response('tasks/task_list.html', {'tasks':  tasks}, context_instance=RequestContext(request))

def task_list_page(request, pid):
	tasks = Task.objects.filter(page=pid).values('id', 'status', 'name', 'modification_date', 'page', 'author').order_by('-modification_date')
	if len(tasks) == 0:
		return render_to_response('pages/bug.html', {'bug': _('There are no tasks for this page. To create a new task click <a href="/tasks/add/') + str(pid) + _('/">here</a>')}, context_instance=RequestContext(request))
	for i in tasks:
		p = Page.objects.get(id=i['page'])
		if p.slug == 'index':
			i['page'] = _('Global Task')
		else:
			i['page'] = '<a href="/w/p/' + p.slug + '/">' + p.title + '</a>'
	return render_to_response('tasks/task_list.html', {'tasks':  tasks}, context_instance=RequestContext(request))

def task_show(request, task_id):
	try:
		task = Task.objects.get(id=task_id)
	except:
		return render_to_response('pages/bug.html', {'bug': _('There is no Task with this ID')}, context_instance=RequestContext(request))
	from boxcomments.models import Comment
	com = Comment.objects.filter(appid = task_id, apptype = 3).order_by('-date')
	if request.user.is_authenticated():
		staff = request.user.is_staff
	else:
		staff = False
	return render_to_response('tasks/task_show.html', {'task': task, 'com': com, 'staff': staff}, context_instance=RequestContext(request))

def task_add(request, pid=False):
	if request.user.is_authenticated():
		manipulator = Task.AddManipulator()
		page = False
		if pid:
			try:
				page = Page.objects.values('id', 'title', 'slug').get(id=pid)
			except Page.DoesNotExist:
				pass
		if request.POST:
			data = request.POST.copy()
			if page:
				data['page'] = page['id']
			errors = manipulator.get_validation_errors(data)
			if not errors:
				data['author'] = str(request.user)
				
				from re import findall, MULTILINE
				import base64
				from stripogram import html2safehtml
				tags = findall( r'(?xs)\[code\](.*?)\[/code\]''', data['text'], MULTILINE)
				for i in tags:
					data['text'] = data['text'].replace('[code]'+i+'[/code]', '[code]'+base64.encodestring(i)+'[/code]')
				data['text'] = html2safehtml(data['text'] ,valid_tags=('b', 'a', 'i', 'br', 'p', 'u', 'img', 'li', 'ul', 'ol', 'center', 'sub', 'sup', 'cite', 'blockquote'))
				tags = findall( r'(?xs)\[code\](.*?)\[/code\]''', data['text'], MULTILINE)
				for i in tags:
					data['text'] = data['text'].replace('[code]'+i+'[/code]', '[code]'+base64.decodestring(i)+'[/code]')
				data['name'] = html2safehtml(data['name'] ,valid_tags=())
				
				if len(data['text']) < 5 or len(data['name']) < 5:
					errors['name'] = [_('Minimum lenght for Task Name and Task Text is 5 characters')]
					form = forms.FormWrapper(manipulator, data, errors)
					return render_to_response('tasks/task_add.html', {'form':form}, context_instance=RequestContext(request))
				manipulator.do_html2python(data)
				data['status'] = True
				new = manipulator.save(data)
				from django.core.mail import mail_admins
				s = Site.objects.get(id=settings.SITE_ID)
				mail_admins('Zadanie Dodane', "Dodano zadanie: http://www." + str(s) + "/tasks/show/" + str(new.id) + '/', fail_silently=True)
				return HttpResponseRedirect('/tasks/list/')
			else:
				form = forms.FormWrapper(manipulator, data, errors)
				return render_to_response('tasks/task_add.html', {'form':form, 'page':page}, context_instance=RequestContext(request))
		else:
			errors = {}
			data = {}
		form = forms.FormWrapper(manipulator, data, errors)
		return render_to_response('tasks/task_add.html', {'form':form, 'page':page}, context_instance=RequestContext(request))
	else:
		return render_to_response('pages/bug.html', {'bug': _('You don\'t have the permissions to add tasks. Are you logged in?')}, context_instance=RequestContext(request))

def zadania(request):
	return render_to_response('tasks/zadania.html', context_instance=RequestContext(request))