from django.shortcuts import render_to_response
from userpanel.models import *
from django import forms
from django.core import validators
from django.http import HttpResponseRedirect, HttpResponse
from django.conf import settings
from stripogram import html2safehtml
from django.contrib.auth.models import User, Group

# main user panel
def user_panel(request):
	if not request.user.is_authenticated():
		return HttpResponseRedirect("/user/login/")
	return render_to_response('userpanel/panel.html', {'sid': settings.SITE_ID})

# list of users
def userlist(request):
	users = User.objects.all()
	return render_to_response('userpanel/userlist.html', {'users': users, 'sid': settings.SITE_ID})
##################################
# Register a user and add it to default user group
##################################
class RegisterForm(forms.Manipulator):
	def __init__(self):
		self.fields = (forms.TextField(field_name="login", length=20, maxlength=200, is_required=True, validator_list=[validators.isAlphaNumeric, self.size3, self.freelogin]),
		forms.PasswordField(field_name="password1", length=20, maxlength=200, is_required=True, validator_list=[validators.isAlphaNumeric, self.size4, self.equal]),
		forms.PasswordField(field_name="password2", length=20, maxlength=200, is_required=True, validator_list=[validators.isAlphaNumeric, self.size4]),
		forms.TextField(field_name="imgtext", is_required=True, validator_list=[self.hashcheck], length=20),
		forms.TextField(field_name="imghash", is_required=True, length=20),
		forms.EmailField(field_name="email", is_required=True, length=20, validator_list=[self.freemail]),)
	def hashcheck(self, field_data, all_data):
		import sha
		SALT = settings.SECRET_KEY[:20]
		if not all_data['imghash'] == sha.new(SALT+field_data.upper()).hexdigest():
			raise validators.ValidationError(_("Captcha Error."))
	def size3(self, field_data, all_data):
		if len(field_data) < 4:
			raise validators.ValidationError(_("Login to short"))
	def size4(self, field_data, all_data):
		if len(field_data) < 5:
			raise validators.ValidationError(_("Password to short"))
	def equal(self, field_data, all_data):
		if all_data['password2'] != field_data:
			raise validators.ValidationError(_("Passwords do not match"))
	def freelogin(self, field_data, all_data):
		try:
			User.objects.get(username=field_data)
		except:
			pass
		else:
			raise validators.ValidationError(_("Login already taken"))
	def freemail(self, field_data, all_data):
		try:
			User.objects.get(email=field_data)
		except:
			pass
		else:
			raise validators.ValidationError(_('Email already taken'))


# register user
def register(request):
	from django.contrib.auth import authenticate, login
	# captcha image creation
	from random import choice
	import Image, ImageDraw, ImageFont, sha
	# create a 5 char random strin and sha hash it
	imgtext = ''.join([choice('QWERTYUOPASDFGHJKLZXCVBNM') for i in range(5)])
	SALT = settings.SECRET_KEY[:20]
	imghash = sha.new(SALT+imgtext).hexdigest()
	# create an image with the string
	im=Image.open(settings.MEDIA_ROOT + '/bg.jpg')
	draw=ImageDraw.Draw(im)
	font=ImageFont.truetype(settings.MEDIA_ROOT + '/SHERWOOD.TTF', 26)
	draw.text((5,5),imgtext, font=font, fill=(100,100,50))
	im.save(settings.MEDIA_ROOT + '/captcha/' + imghash + '.jpg',"JPEG")
	
	manipulator = RegisterForm()
	if request.POST:
		data = request.POST.copy()
		
		errors = manipulator.get_validation_errors(data)
		if not errors:
			data['email'] = html2safehtml(data['email'] ,valid_tags=())
			try:
				user = User.objects.create_user(data['login'], data['email'], data['password1'])
			except Exception:
				data['imgtext'] = ''
				form = forms.FormWrapper(manipulator, data, errors)
				return render_to_response('userpanel/register.html', {'error': True, 'form': form, 'sid': settings.SITE_ID})
			else:
				user.save()
				user = authenticate(username=data['login'], password=data['password1'])
				if user is not None:
					login(request, user)
					#user.groups.add(Group.objects.get(name='users'))
				from os import remove
				remove(settings.MEDIA_ROOT + '/captcha/' + data['imghash'] + '.jpg')
				return HttpResponseRedirect("/user/")
		else:
			data['imgtext'] = ''
			form = forms.FormWrapper(manipulator, data, errors)
			return render_to_response('userpanel/register.html', {'error': True, 'hash': imghash, 'form': form, 'sid': settings.SITE_ID})
	else:
		errors = data = {}
	form = forms.FormWrapper(manipulator, data, errors)
	return render_to_response('userpanel/register.html', {'hash': imghash, 'form': form, 'sid': settings.SITE_ID})

##############################################
# Login / logout user
##############################################
class LoginForm(forms.Manipulator):
	def __init__(self):
		self.fields = (forms.TextField(field_name="login", length=30, maxlength=200, is_required=True),
		forms.PasswordField(field_name="password", length=30, maxlength=200, is_required=True),)

def loginlogout(request):
	from django.contrib.auth import authenticate, login
	if not request.user.is_authenticated():
		manipulator = LoginForm()
		# log in user
		if request.POST:
			data = request.POST.copy()
			errors = manipulator.get_validation_errors(data)
			if not errors:
				manipulator.do_html2python(data)
				user = authenticate(username=data['login'], password=data['password'])
				if user is not None:
					login(request, user)
					return HttpResponseRedirect("/user/")
				else:
					form = forms.FormWrapper(manipulator, data, errors)
					return render_to_response('userpanel/login.html', {'loginform': True, 'error': True, 'form': form, 'sid': settings.SITE_ID})
		# no post data, show the login form
		else:
			errors = data = {}
		form = forms.FormWrapper(manipulator, data, errors)
		if request.GET and request.GET.has_key('b'):
			return render_to_response('userpanel/login.html', {'loginform': True, 'form': form, 'sid': settings.SITE_ID, 'reset': True})
		return render_to_response('userpanel/login.html', {'loginform': True, 'form': form, 'sid': settings.SITE_ID})
	else:
		# user authenticated
		if request.GET:
			# logout user
			data = request.GET.copy()
			if data.has_key('log'):
				from django.contrib.auth import logout
				logout(request)
				return HttpResponseRedirect("/user/")
		return HttpResponseRedirect("/user/")

##############################
# Send messages - emails
##############################
class PMessage(forms.Manipulator):
	def __init__(self):
		self.fields = (forms.TextField(field_name="subject", length=30, maxlength=200, is_required=True),
		forms.LargeTextField(field_name="contents", is_required=True),)

def send_pmessage(request, target_user):
	if request.user.is_authenticated() and str(request.user) != str(target_user) and len(str(target_user)) > 0 and str(target_user) != 'AnonymousUser':
		ruser = User.objects.get(username=str(target_user))
		try:
			ruser_profile = Profile.objects.get(username=ruser)
		except:
			mess = False
		else:
			mess = ruser_profile.use_messages
		if not mess:
			return render_to_response('pages/bug.html', {'bug': _('This user doesn\'t want to receive any messages')})
		manipulator = PMessage()
		if request.POST:
			new_data = request.POST.copy()
			errors = manipulator.get_validation_errors(new_data)
			if not errors:
				manipulator.do_html2python(new_data)
				from django.core.mail import send_mail
				send_mail(new_data['subject'], new_data['contents'], request.user.email, [ruser.email], fail_silently=False)
				return HttpResponseRedirect("/user/")
		else:
			errors = new_data = {}
		form = forms.FormWrapper(manipulator, new_data, errors)
		return render_to_response('userpanel/pmessage.html', {'form': form, 'sid': settings.SITE_ID})
	else:
		return HttpResponseRedirect("/user/")

def edit_profile_pmessage(request):
	if request.user.is_authenticated():
		try:
			p = Profile.objects.get(username=request.user)
		except:
			p = Profile(username=request.user)
			p.save()
		if request.POST:
			data = request.POST.copy()
			if not data.has_key('use_messages'):
				data['use_messages'] = False
			else:
				data['use_messages'] = True
			p.use_messages = data['use_messages']
			p.save()
			return HttpResponseRedirect("/user/")
		return render_to_response('userpanel/profile_pmessage.html', {'p': p})
	else:
		return HttpResponseRedirect("/user/login/")
##############################
# reset password
##############################
class HMessage(forms.Manipulator):
	def __init__(self):
		self.fields = (forms.TextField(field_name="login", length=30, maxlength=200, is_required=True),
		forms.EmailField(field_name="email", is_required=True, length=20),)

def send_hmessage(request):
	if request.user.is_authenticated():
		return HttpResponseRedirect('/user/password_reset/')
	manipulator = HMessage()
	if request.POST:
		new_data = request.POST.copy()
		errors = manipulator.get_validation_errors(new_data)
		if not errors:
			manipulator.do_html2python(new_data)
			if new_data['login'].lower() == 'riklaunim' or new_data['email'].lower() == 'riklaunim@gmail.com':
				return render_to_response('pages/bug.html', {'bug': _('ROTFL LOL OMG CSS :)')})
			try:
				u = User.objects.get(username=new_data['login'], email = new_data['email'])
			except:
				return render_to_response('pages/bug.html', {'bug': _('No such user account')})
			else:
				a = User.objects.make_random_password(length=6, allowed_chars='abcdefghjkmnpqrstuvwxyz23456789')
				u.set_password(a)
				from django.core.mail import send_mail
				send_mail(_('Password reset'), _('Your new password on rk.edu.pl sites is: ') + a, 'riklaunim@gmail.com', [new_data['email']], fail_silently=False)
				u.save()
			return HttpResponseRedirect("/user/login/?b=ok")
	else:
		errors = new_data = {}
	form = forms.FormWrapper(manipulator, new_data, errors)
	return render_to_response('userpanel/hmessage.html', {'form': form})

##############################
# change password
##############################
class ZMessage(forms.Manipulator):
	def __init__(self):
		self.fields = (forms.TextField(field_name="haslo1", length=30, maxlength=200, is_required=True, validator_list=[validators.isAlphaNumeric, self.size4, self.equal]),
		forms.TextField(field_name="haslo2", length=30, maxlength=200, is_required=True),)
	def equal(self, field_data, all_data):
		if all_data['haslo2'] != field_data:
			raise validators.ValidationError(_('Passwords do not match'))
	def size4(self, field_data, all_data):
		if len(field_data) < 5:
			raise validators.ValidationError(_('To short'))

def send_zmessage(request):
	if request.user.is_authenticated():
		if request.user.is_staff:
			return render_to_response('pages/bug.html', {'bug': _('ROTFL LOL OMG CSS :)')})
		manipulator = ZMessage()
		if request.POST:
			new_data = request.POST.copy()
			errors = manipulator.get_validation_errors(new_data)
			if not errors:
				manipulator.do_html2python(new_data)
				request.user.set_password(new_data['haslo1'])
				request.user.save()
				return HttpResponseRedirect("/user/")
		else:
			errors = new_data = {}
		form = forms.FormWrapper(manipulator, new_data, errors)
		return render_to_response('userpanel/zmessage.html', {'form': form})
	else:
		return HttpResponseRedirect("/user/login/")